@@ -8,6 +8,8 @@
 #include <stdio.h>
 #include <security/pam_appl.h>
 
+#include "cfg.h"
+
 #define BUFSIZE 1024
 #define MAX_DEVS 24
 #define DEFAULT_AUTHFILE_DIR_VAR "XDG_CONFIG_HOME"
@@ -16,33 +18,12 @@
 #define DEFAULT_AUTHFILE_DIR ".config"
 #define DEFAULT_AUTHFILE_DIR_SSH ".ssh"
 #define DEFAULT_AUTHPENDING_FILE_PATH "/var/run/user/%d/pam-u2f-authpending"
-#define DEFAULT_PROMPT "Insert your U2F device, then press ENTER."
-#define DEFAULT_CUE "Please touch the device."
+#define DEFAULT_PROMPT "Insert your FIDO authenticator, then press ENTER."
+#define DEFAULT_CUE "Please touch the FIDO authenticator."
 #define DEFAULT_ORIGIN_PREFIX "pam://"
 #define SSH_ORIGIN "ssh:"
 
-typedef struct {
-  unsigned max_devs;
-  int manual;
-  int debug;
-  int nouserok;
-  int openasuser;
-  int alwaysok;
-  int interactive;
-  int cue;
-  int nodetect;
-  int userpresence;
-  int userverification;
-  int pinverification;
-  int sshformat;
-  const char *auth_file;
-  const char *authpending_file;
-  const char *origin;
-  const char *appid;
-  const char *prompt;
-  const char *cue_prompt;
-  FILE *debug_file;
-} cfg_t;
+#define DEVLIST_LEN 64
 
 typedef struct {
   char *publicKey;
@@ -64,6 +45,7 @@ char *converse(pam_handle_t *pamh, int echocode, const char *prompt);
 int random_bytes(void *, size_t);
 int cose_type(const char *, int *);
 const char *cose_string(int);
+char *expand_variables(const char *, const char *);
 
 #if !defined(HAVE_EXPLICIT_BZERO)
 void explicit_bzero(void *, size_t);
